import java.util.*;

/**
 * Clase que representa un mapa con troceado abierto
 */
public class MapaAbierto<K,V>
{
    // atributos privados
    private ListaEnlazadaSimple<Entry<K,V>>[] tabla; 

    private static class Entry<K,V> {
        private K clave;
        private V valor;

        public Entry(K clave, V valor) {
            this.clave=clave;
            this.valor=valor;
        }
    
        public boolean equals(Object otra) {
            if (otra instanceof Entry) {
                return clave.equals(((Entry<K,V>)otra).clave);
            } else {
                return false;
            }
        }
    }
  
    /**
     * Constructor al que se le pasa el tamano de la tabla
     */
    public MapaAbierto(int tamano) {
        tabla =  new ListaEnlazadaSimple[tamano];
        for (int i = 0; i < tamano; i++) {
            tabla[i] = new ListaEnlazadaSimple<Entry<K,V>>();
        }
    }

    private int valorHash(K clave) {
        return clave.hashCode() % tabla.length;
    }

    /**
     * Asigna un valor a una clave
     */
    public V put(K clave, V valor) {
        int hashVal = valorHash(clave);
        Entry<K,V> nuevo= new Entry<K,V>(clave,valor);
        Entry<K,V> e=tabla[hashVal].find(nuevo);
        if (e==null) {
            tabla[hashVal].addFirst(nuevo); 
            return null;
        } else {
            V viejo=e.valor;
            e.valor=valor; 
            return viejo;
        }
    }


    /**
     * Elimina la clave indicada del mapa, retornando su valor asociado
     */
    public V remove(K clave) {
        int hashVal = valorHash(clave);
        Entry<K,V> nuevo= new Entry<K,V>(clave,null);
        Entry<K,V> e=tabla[hashVal].find(nuevo);
        if (e==null) {
            return null;
        } else {
            V valor=e.valor;
            tabla[hashVal].remove(nuevo);
            return valor;
        }
    }

    /**
     * Obtiene el valor asociado a una clave
     */
    public V get(K clave) {
        int hashVal = valorHash(clave);
        Entry<K,V> nuevo= new Entry<K,V>(clave,null);
        Entry<K,V> e=tabla[hashVal].find(nuevo);
        if (e==null) {
            return null;
        } else {
            return e.valor;
        }    
    }

    /**
     * Averigua si la clave indicada esta en el mapa
     */
    boolean containsKey(K clave) {
        int hashVal = valorHash(clave);
        Entry<K,V> nuevo= new Entry<K,V>(clave,null);
        Entry e=tabla[hashVal].find(nuevo);
        return e!=null;
    }
  
}
