import java.util.*;
/**
 * Implementacion de un mapa usando troceado cerrado
 */
public class MapaCerrado <K,V>
{
    /**
     * Clase Entry, que define una pareja clave-valor
     * y anota si se ha borrado o no
     */
    private static class Entry<K,V> {
        private K clave;
        private V valor;
        boolean borrado;

        public Entry(K clave, V valor) {
            this.clave = clave;
            this.valor = valor;
            this.borrado=false;
        }
    
        public boolean equals(Object otro) {
            if (otro instanceof Entry) {
                return clave.equals(((Entry<K,V>)otro).clave);
            } else {
                return false;
            }
        }   
    }

    // atributos privados de la clase MapaCerrado
    private Object[] tabla;
    private int num;
  
    /**
     * Constructor al que se le indica el tamano inicial de la tabla
     */
    public MapaCerrado(int tamanoInicial) {
        tabla = new Object[tamanoInicial];
        num=0;
    }
 
    // funcion hash
    private int valorHash(K clave) {
        return clave.hashCode() % tabla.length;
    }
  
    // funcion para buscar una celda
    private int buscaIndice(Entry<K,V> e) {
        int pos = valorHash(e.clave);
        // bucle hasta encontrar una celda vacia
        while (tabla[pos] != null && !e.equals(tabla[pos])) {
            pos++; // ir a la siguiente celda
            pos = pos % tabla.length; // ir al principio si hace falta
        }
        return pos;
    }

    /**
     * Asigna un valor a una clave
     */
    public V put(K clave, V valor) {
        Entry<K,V> nuevo= new Entry<K,V>(clave,valor);
        int pos=buscaIndice(nuevo);
        V viejo=null;
        if (tabla[pos]!=null && !((Entry<K,V>)tabla[pos]).borrado) {
            viejo=((Entry<K,V>)tabla[pos]).valor;
        }
        if (tabla[pos]==null) {
            num++;
        }
        tabla[pos] = nuevo;
        if (num>=tabla.length/2) {
            // recrear la tabla
            Object[] tablaAntigua=tabla;
            tabla= new Object[tablaAntigua.length*2];
            num=0;
            for (int i=0; i<tablaAntigua.length; i++) {
                Entry<K,V> e=(Entry<K,V>)tablaAntigua[i];
                if (e!=null && !e.borrado) {
                    put(e.clave, e.valor);
                }
            }
        }
        return viejo;
    }

    /**
     * Elimina la clave indicada del mapa, retornando su valor asociado
     */

    public V remove(K clave) {
        Entry<K,V> e=new Entry<K,V>(clave,null);
        int pos=buscaIndice(e);
        if (tabla[pos]==null || ((Entry<K,V>)tabla[pos]).borrado) {
            return null;
        } else {
            V valor=((Entry<K,V>)tabla[pos]).valor;
            ((Entry<K,V>)tabla[pos]).borrado=true;
            return valor;
        }
    }

    /**
     * Obtiene el valor asociado a una clave
     */
    public V get(K clave) {
        Entry<K,V> e=new Entry<K,V>(clave,null);
        int pos=buscaIndice(e);
        if (tabla[pos]==null || ((Entry<K,V>)tabla[pos]).borrado) {
            return null;
        } else {
            return ((Entry<K,V>)tabla[pos]).valor;
        }
    }

    /**
     * Averigua si la clave indicada esta en el mapa
     */
    boolean containsKey(K clave) {
        Entry<K,V> e=new Entry<K,V>(clave,null);
        int pos=buscaIndice(e);
        return tabla[pos]!=null && !((Entry<K,V>)tabla[pos]).borrado;
    }
}
