
/**
 * Clase que ofrece una implementacion minima de una
 * cola de prioridad usando un monticulo binario
 */
public class MonticuloBinario <E extends Comparable<E>>
{

    // atributos de la clase
    private Object[] tabla;
    private int num;
    
    /**
     * Constructor al que se le pasa el tamano de la tabla
     * y un valor menor que todos los demas
     */
    public MonticuloBinario(int tamano, E menosInfinito) {
        tabla = new Object[tamano];
        tabla[0]=menosInfinito;
        num=0;
    }
    
    /**
     * Inserta un nuevo elemento en el monticulo
     */
    public void inserta(E elem) throws NoCabe {
        if (num==tabla.length-1) {
            throw new NoCabe();   
        }
        num++;
        int hueco=num;
        // reflotar comparando con el padre
        while(elem.compareTo((E)tabla[hueco/2])<0) {
            tabla[hueco]=tabla[hueco/2];
            hueco=hueco/2;
        }
        tabla[hueco]=elem;
    }
    
    /**
     * Elimina el elemento menor, y lo retorna
     */
    public E elimina() throws NoHay {
        E borrado=primero();
        E x=(E) tabla[num];
        num--;
        int hueco=1;
        
        // hundir el hueco
        boolean acabar=false;
        while (! acabar && hueco*2<=num) {
            int hijo=hueco*2;
            if (hijo!=num && ((E)tabla[hijo+1]).compareTo
                ((E)tabla[hijo])<0)
                {
                    hijo++; 
                }
            if (((E)tabla[hijo]).compareTo(x)<0) {
                tabla[hueco]=tabla[hijo];
                hueco=hijo;
            } else {
                acabar=true;   
            }
        }
        tabla[hueco]=x;
        return borrado;
    }
    
    /**
     * Retorna el elemento minimo sin borrarlo
     */
    public E primero() throws NoHay {
        if (estaVacio()) {
            throw new NoHay();
        } else {
            return (E) tabla[1];
        }
    }
    
    /**
     * Indica si el monticulo esta vacio
     */
    public boolean estaVacio() {
        return num==0;
    }
    
    /**
     * Vacia el monticulo
     */
    public void hazNulo() {
        num=0;
    }
}
